;****************************************************
;*File Name:	RC5_Send.asm						*
;*Title:		RC5 IR Remote Control Transmitter	*
;*Version:		1.0									*
;*Author:		R. Shams							*
;*Support Mail:	r.shamss@gmail.com					*
;*Target MCU:	ATTiny13							*
;*Editor:		AVR Studio 4.11						*
;*Assembler:	AVR Assembler Version 2				*
;* 													*
;*The timing is adapted for 9.6 MHz crystal			*
;****************************************************
.include "tn13def.inc"
.cseg
.org 0
rjmp reset		; Reset Handler
reti			; IRQ0 Handler
rjmp _pcint0	; PCINT0 Handler
reti			; Timer0 Overflow Handler
reti			; EEPROM Ready Handler
reti			; Analog Comparator Handler
reti			; Timer0 CompareA Handler
reti			; Timer0 CompareB Handler
reti			; Watchdog Interrupt Handler
reti			; ADC Conversion Handler
;
;****************************************************
reset:
	; Set Stack Pointer to top of RAM
	ldi r16,low(ramend)
	out spl,r16
	; Config PortB
	ldi r16,$01
	out ddrb,r16
	;Enable Puulups
	ldi r16,$1e
	out portb,r16
	;Timer/Counter  Output Compare Register A
	;Will Generate 37.5KHz Carrier
	ldi r16,$80
	out ocr0a,r16
	;Carrier off
	ldi r16,$c2
	out tccr0a,r16
	;Timer on
	ldi r16,$01
	out tccr0b,r16
	rcall wait_884us
	;timer off
	ldi r16,$00
	out tccr0b,r16
	;(MCUCR) mcu control register(sleep mode,enable sleep)
	ldi r16,$30
	out mcucr,r16
	;addres
	ldi r19,13
	;(gimsk) General Interrupt Mask Register
	ldi r16,$20
	out gimsk,r16
	;(PCMSK) Pin Change Mask Register
	ldi r16,$1e
	out pcmsk,r16
	sei
;
;***********************Main*************************
main:
	sleep
rjmp main
;********************End Program*********************
;
_pcint0:
	sbis pinb,1
		rjmp key_1
	sbis pinb,2
		rjmp key_2
	sbis pinb,3
		rjmp key_3
	sbis pinb,4
		rjmp key_4
;----------------------------------------------------
return:
	;wait until to all key no press
	sbis pinb,1
		rjmp return
	sbis pinb,2
		rjmp return
	sbis pinb,3
		rjmp return
	sbis pinb,4
		rjmp return
	;Toogle Bit
	com r0
	;reset flag
	ldi r16,$20
	out gifr,r16
reti
;
;----------------------------------------------------
key_1:
	rcall wait_30ms
key_11:
	sbic pinb,1
		rjmp return
	;command
	ldi r18,41
	rcall transfer
	rjmp key_11
;
key_2:
	rcall wait_30ms
key_22:
	sbic pinb,2
		rjmp return
	;command
	ldi r18,14
	rcall transfer
	rjmp key_22
;
key_3:
	rcall wait_30ms
key_33:
	sbic pinb,3
		rjmp return
	;command
	ldi r18,53
	rcall transfer
	rjmp key_33
;
key_4:
	rcall wait_30ms
key_44:
	sbic pinb,4
		rjmp return
	;command
	ldi r18,18
	rcall transfer
	rjmp key_44
;
;****************************************************
transfer:
	;Timer on
	ldi r16,$01
	out tccr0b,r16
	;Send first start bit
	rcall logic_1
	;Send second start bit
	rcall logic_1
	;Send toggle bit
	sbrc r0,0
		rcall logic_1
	sbrs r0,0
		rcall logic_0
	;Send 5 bit address, msb first
	;Bit 4
	sbrc r19,4
		rcall logic_1
	sbrs r19,4
		rcall logic_0
	;Bit 3
	sbrc r19,3
		rcall logic_1
	sbrs r19,3
		rcall logic_0
	;Bit 2
	sbrc r19,2
		rcall logic_1
	sbrs r19,2
		rcall logic_0
	;Bit 1
	sbrc r19,1
		rcall logic_1
	sbrs r19,1
		rcall logic_0
	;Bit 0
	sbrc r19,0
		rcall logic_1
	sbrs r19,0
		rcall logic_0
	;Send 6 bit command, msb first
	;Bit 5
	sbrc r18,5
		rcall logic_1
	sbrs r18,5
		rcall logic_0
	;Bit 4
	sbrc r18,4
		rcall logic_1
	sbrs r18,4
		rcall logic_0
	;Bit 3
	sbrc r18,3
		rcall logic_1
	sbrs r18,3
		rcall logic_0
	;Bit 2
	sbrc r18,2
		rcall logic_1
	sbrs r18,2
		rcall logic_0
	;Bit 1
	sbrc r18,1
		rcall logic_1
	sbrs r18,1
		rcall logic_0
	;Bit 0
	sbrc r18,0
		rcall logic_1
	sbrs r18,0
		rcall logic_0
	;Carrier off
	ldi R16,$C2
	Out Tccr0a,R16
	;frame gap delay
	rcall wait_87ms
	;Timer off
	ldi r16,$00
	out tccr0b,r16
ret
;----------------------------------------------------
logic_1:
	;Carrier off
	LDI R16,$C2
	Out Tccr0a,R16
	;Waitus 883
	rcall wait_883us
	;Carrier on
	ldi r16,$42
	out tccr0a,r16
	;Waitus 884
	rcall wait_884us
ret
;----------------------------------------------------
logic_0:
	;Carrier on
	ldi r16,$42
	out tccr0a,r16
	;Waitus 883
	rcall wait_883us
	;Carrier off
	ldi R16,$C2
	Out Tccr0a,R16
	;Waitus 884
	rcall wait_884us
ret
;
;----------------------------------------------------
wait_883us:
	; ============================= 
	;    delay loop generator 
	;     8477 cycles:
	; ----------------------------- 
	; delaying 8475 cycles:
	           ldi  R20, $19
	WGLOOP00:  ldi  R21, $70
	WGLOOP01:  dec  R21
	           brne WGLOOP01
	           dec  R20
	           brne WGLOOP00
	; ----------------------------- 
	; delaying 2 cycles:
	          nop
	          nop
	; =============================  
ret
;----------------------------------------------------
wait_884us:
	; ============================= 
	;    delay loop generator 
	;     8486 cycles:
	; ----------------------------- 
	; delaying 8484 cycles:
	           ldi  R20, $0E
	WGLOOP10:  ldi  R21, $C9
	WGLOOP11:  dec  R21
	           brne WGLOOP11
	           dec  R20
	           brne WGLOOP10
	; ----------------------------- 
	; delaying 2 cycles:
	          nop
	          nop
	; =============================  
ret
;----------------------------------------------------
wait_87ms:
	; ============================= 
	;    delay loop generator 
	;     835200 cycles:
	; ----------------------------- 
	; delaying 835197 cycles:
	          ldi  R20, $0B
	WGLOOP0:  ldi  R21, $6F
	WGLOOP1:  ldi  R22, $E3
	WGLOOP2:  dec  R22
	          brne WGLOOP2
	          dec  R21
	          brne WGLOOP1
	          dec  R20
	          brne WGLOOP0
	; ----------------------------- 
	; delaying 3 cycles:
	          ldi  R20, $01
	WGLOOP3:  dec  R20
	          brne WGLOOP3
	; =============================  
ret
;
;****************************************************
wait_30ms:
	; ============================= 
	;    delay loop generator 
	;     288000 cycles:
	; ----------------------------- 
	; delaying 287994 cycles:
	          ldi  R20, $02
	WGLOOP4:  ldi  R21, $CE
	WGLOOP5:  ldi  R22, $E8
	WGLOOP6:  dec  R22
	          brne WGLOOP6
	          dec  R21
	          brne WGLOOP5
	          dec  R20
	          brne WGLOOP4
	; ----------------------------- 
	; delaying 6 cycles:
	          ldi  R20, $02
	WGLOOP7:  dec  R20
	          brne WGLOOP7
	; ============================= 
ret





