/*********************************************
Chip type           : AT90S2313
Clock frequency     : 4.000000 MHz
Memory model        : Tiny
External SRAM size  : 0
Data Stack size     : 32
*********************************************/
#include <90s2313.h>
#include <delay.h>

#define LATCH	PORTB.0
#define CLEAR	PORTD.6
#define OE		PORTD.5
#define OCF1A   0x40
#define TOIE0   0x02

unsigned char 	cntrh = 0,Khz,Point;
unsigned char 	disp_value[6] = {0,0,0,0,0,0};
unsigned long	counter=0;

void display(void);
void convert(void);
unsigned long Read_Counter(void);
void Capture(unsigned int Data);

flash unsigned char SEG[] = {0b0111111,0b0000110,0b1011011,0b1001111,0b1100110,
							 0b1101101,0b1111101,0b0000111,0b1111111,0b1101111};

interrupt [TIM0_OVF] void timer0_ovf_isr(void)
{
	cntrh++;
}

void main(void)
{
	// Func0=Out Func1=Out Func2=Out Func3=Out Func4=Out Func5=Out Func6=Out Func7=Out 
	// State0=T State1=T State2=T State3=T State4=T State5=T State6=T State7=T 
	PORTB=0x00;
	DDRB=0xFF;
	
	// Func0=Out Func1=Out Func2=Out Func3=Out Func4=Out Func5=In Func6=Out 
	// State0=0 State1=0 State2=0 State3=0 State4=0 State5=T State6=0 
	PORTD=0x00;
	DDRD=0x5F;
	
	// Timer/Counter 0 initialization
	// Clock source: T0 pin Falling Edge
	TCCR0=0x06;
	TCNT0=0x00;
	
	// Timer/Counter 1 initialization
	// Clock source: System Clock
	// Clock value: 62.500 kHz
	// Mode: CTC top=OCR1A
	// OC1 output: Discon.
	// Noise Canceler: Off
	// Input Capture on Falling Edge
	TCCR1A=0x00;
	TCCR1B=0x0B;
	TCNT1H=0x00;
	TCNT1L=0x00;
	OCR1H=0x00;
	OCR1L=0x00;
	
	// Timer(s)/Counter(s) Interrupt(s) initialization
	TIMSK=0x02;

	// Global enable interrupts
	#asm("sei")

	while (1)
	{
		unsigned char dp,ms;
		// try a capture at min gate
		Capture(500);		// 1 mS
		// get the data		
		counter = Read_Counter();
		dp = 1;		// decimal point
		ms = 1;		// indicate KHz
		
		if (counter < 90000)		// less than 10 MHz
		{
			// try a capture at next gate value
			Capture(5000);		// 10 mS
			// get the data		
			counter = Read_Counter();
			dp = 2;		// decimal point
			ms = 1;		// indicate KHz

			if (counter < 90000)	// less than 1 MHz
			{
				// try a capture at next gate value
				Capture(50000);	// 100 mS
				// get the data		
				counter = Read_Counter();
				dp = 3;		// decimal point
				ms = 1;		// indicate kHz
			
				if (counter < 90000)	// less than 100 kHz
				{
					// try a capture at next gate value
					Capture(15625);		// 1 S
					// get the data		
					counter = Read_Counter();
					dp = 0;		// decimal point
					ms = 0;		// indicate Hz
				}
			}
		}
		
		convert();

		// set display variables
		Point = dp;
		Khz = ms;

	}
}

void display(void)
{
	unsigned char Hi,Lo;
	static unsigned char Segment=0;
	
 	Segment++;
 	if(Segment==6) Segment=0;
 	
	// Turn off display
	PORTB = 0xFF;
	
	Lo = SEG[disp_value[Segment]];
	
	if (Segment == 0 )
	{
		if (Khz)
			Lo |= 0x80;
	}
	else if (Point == Segment )
		Lo |= 0x80;

	Hi = Lo & 0xF0;					// Hi nibble
	Lo = Lo & 0x0F;					// Lo nibble

	PORTB = Hi | 0x0F;				// Hi nibble
	PORTD = (PORTD & 0xF0) | Lo;	// Lo nibble

	PORTB = ((PORTB & 0xf8) | Segment)^1;

 	delay_us(10);
}

void convert(void)
{
	unsigned char j;
	// convert counter to bcd code
	for (j=0;j<6;j++)
	{
		disp_value[j] = counter % 10;
		counter /= 10;
		display();					//refresh display
	}
}

unsigned long Read_Counter(void)
{
	unsigned long Value;

	PORTB = PORTB | 0x07;			// turn off display
	delay_us(10);
	LATCH = 0;						//latch counter
	delay_us(10);
	LATCH = 1;
		
	//Set Port B & C direction
	DDRB = 0x0F;
	DDRD = 0xE0;

	OE = 0;							//Enable Output of Counter
	delay_us(10);

	//Read Internal counter
	Value = TCNT0;
	Value *= 256;					// 8 bit shift

	//Read external counter
	Value |= PINB & 0xF0;
	Value |= PIND & 0x0F;
	
	//Disable Output of Counter
	OE = 1;

	//Clear external counter
	CLEAR = 0;	
	delay_us(10);
	CLEAR = 1;
	

	//Set Port B & C direction
	DDRB = 0xFF;
	DDRD = 0xEF;

	Value += ((unsigned long)cntrh << 16);
	cntrh = 0;
	TCNT0 = 0;
	return Value;
}

void Capture(unsigned int Data)
{

	CLEAR = 0;		// clear external counters
	
	TCNT1H = 0;		// clear timers
	TCNT1L = 0;
	TCNT0  = 0;
	cntrh = 0;

	CLEAR = 1;		// remove clear

	OCR1H = Data >> 8;	// set the compare1 register to the
	OCR1L = Data;	// required value

	TCCR1A = 0x40;		// set OC1 bit to toggle on compare

	TIFR |= OCF1A;		// clear overflov/compare flags

	if (Data == 15625)
		TCCR1B = 0x0C;		// start with fClk/256 (15625 Hz) and compare clear
	else
		TCCR1B = 0x0A;		// start with fClk/8 (500 kHz) and compare clear

	while ( !(TIFR & OCF1A) )	// wait for bit
		display();
				
	TIFR |= OCF1A;			// clear flags
	
	while (!(TIFR & OCF1A) )	// wait again for bit
		display();
	TCCR1B = 0;			// stop timer
	
	// counter input disabled
}
